# Generated by roxygen2: do not edit by hand

export(auto_export_for_mgcv)
export(build_allowed_function)
export(build_allowed_matrix)
export(build_sales_grid)
export(city_abbreviation)
export(compute_intermediate_output)
export(compute_rca_adjustments)
export(compute_sale_age)
export(convert_quarto_file)
export(country_choices)
export(country_schema)
export(default_regproj_root)
export(detect_categoricals)
export(detect_types)
export(earthui_prefs_path)
export(earthui_prefs_read)
export(earthui_prefs_write)
export(export_settings)
export(fit_earth)
export(format_anova)
export(format_model_equation)
export(format_summary)
export(format_variable_importance)
export(generate_quarto_report)
export(get_project_settings)
export(import_data)
export(is_project_dir)
export(launch)
export(list_g_functions)
export(os_detect)
export(plot_actual_vs_predicted)
export(plot_contribution)
export(plot_correlation_matrix)
export(plot_g_contour)
export(plot_g_function)
export(plot_g_persp)
export(plot_partial_dependence)
export(plot_qq)
export(plot_residuals)
export(plot_variable_importance)
export(prepare_report_assets)
export(regproj_flat_segment)
export(regproj_geo_db_connect)
export(regproj_geo_db_path)
export(regproj_in_files)
export(regproj_index_get)
export(regproj_index_path)
export(regproj_index_put)
export(regproj_index_read)
export(regproj_index_write)
export(regproj_last_file_get)
export(regproj_last_file_path)
export(regproj_last_file_set)
export(regproj_list_projects)
export(regproj_parse_flat)
export(regproj_path)
export(regproj_projects_db_connect)
export(regproj_projects_db_path)
export(regproj_reference)
export(render_report)
export(select_sales_grid_comps)
export(set_project_settings)
export(validate_types)
export(write_earth_output)
export(write_fit_log)
importFrom(earth,earth)
importFrom(earth,evimp)
importFrom(ggplot2,.data)
